# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.
#
# This source code is licensed under the terms described in the LICENSE file in
# the root directory of this source tree.
"""
Synthetic Data Kit: A toolkit for preparing synthetic data for LLM fine-tuning
"""

__version__ = "0.0.1"